#include <stdio.h>
#include <math.h>
#include <stdbool.h>
#include <Python.h>

PyObject* square_impl(PyObject* /* unused module reference */, PyObject* o) {
    double x = PyFloat_AsDouble(o);  
    double pow_x = pow(x, 2);
    return PyFloat_FromDouble(pow_x);
}

PyObject* double_impl(PyObject* /* unused module reference */, PyObject* o) {
    double x = PyFloat_AsDouble(o);
    double doubled_x = x * 2;
    return PyFloat_FromDouble(doubled_x);
}

PyObject* multiply_impl(PyObject* /* unused module reference */, PyObject* args) {
    double result = 1.0;
    int arg_count = PyTuple_Size(args);
    for (int i = 0; i < arg_count; ++i) {
        PyObject* o = PyTuple_GetItem(args, i);
        double x = PyFloat_AsDouble(o);
        result *= x;
    }
    return PyFloat_FromDouble(result);
}

static PyMethodDef superfastcode_methods[] = {
    { "fast_square", (PyCFunction)square_impl, METH_O, nullptr },
    { "fast_double", (PyCFunction)double_impl, METH_O, nullptr },
    { "fast_multiply", (PyCFunction)multiply_impl, METH_VARARGS, nullptr },
    { nullptr, nullptr, 0, nullptr }
};

static PyModuleDef test_pyd_module = {
    PyModuleDef_HEAD_INIT,
    "superfastcode",
    "Provides some functions, but faster",
    0,
    superfastcode_methods
};

PyMODINIT_FUNC PyInit_test_pyd() {
    return PyModule_Create(&test_pyd_module);
}

/**
 * build
 * python38 esetén
 * g++ -static-libgcc -static-libstdc++ -lwinpthread -shared -otest_pyd.pyd -I/python38/include test_pyd.cpp /python38/python38.dll
 * python311
 * g++ -static-libgcc -static-libstdc++ -lwinpthread -shared -opy3114/test_pyd.pyd -I./py3114/include test_pyd.cpp python311.dll
 * 
 * ezek csak próbák:
 * g++ -c test_pyd.cpp -I./py3114/include
 * g++ -shared -otest_pyd.pyd -I./py3114/include test_pyd.cpp python311.dll
 * g++ -shared -otest_pyd.pyd -static-libgcc -static-libstdc++ -Wl,-Bstatic,--whole-archive -lwinpthread -Wl,--no-whole-archive test_pyd.o python311.dll
*/